@@ -43,7 +43,7 @@ sub get_utc_time
 sub change_timedate
 {
   my ($time) = @_;
-  my ($command);
+  my (@command);
 
   my $system_table = {
     "Linux"   => "date -u %02d%02d%02d%02d%04d.%02d",
@@ -51,13 +51,13 @@ sub change_timedate
     "SunOS"   => "date -u %02d%02d%02d%02d%04d.%02d",
   };
 
-  $command = sprintf ($$system_table {$Utils::Backend::tool{"system"}},
-                      $$time{"month"} + 1, $$time{"monthday"},
-                      $$time{"hour"},  $$time{"minute"}, 
-                      $$time{"year"},  $$time{"second"});
+  @command = ($$system_table {$Utils::Backend::tool{"system"}},
+              $$time{"month"} + 1, $$time{"monthday"},
+              $$time{"hour"}, $$time{"minute"},
+              $$time{"year"}, $$time{"second"});
 
-  &Utils::Report::do_report ("time_localtime_set", $command);
-  return &Utils::File::run ($command);
+  &Utils::Report::do_report ("time_localtime_set", @command);
+  return &Utils::File::run (@command);
 }
 
 sub set_utc_time
@@ -73,7 +73,7 @@ sub set_utc_time
 
 sub time_sync_hw_from_sys
 {
-  &Utils::File::run ("hwclock --systohc") if ($$tool{"system"} eq "Linux");
+  &Utils::File::run ("hwclock", "--systohc") if ($$tool{"system"} eq "Linux");
   return 0;
 }
 
@@ -116,7 +116,7 @@ sub get_timezone
     $size_test = (stat("$zoneinfo_dir/$zone"))[7];
     if ($size_test eq $size_search)
     {
-      if (!&Utils::File::run ("diff $zoneinfo_dir/$zone $local_time_file"))
+      if (!&Utils::File::run ("diff", "$zoneinfo_dir/$zone", $local_time_file))
       {
         # Found a match.
         last;
@@ -144,7 +144,7 @@ sub set_timezone
     unlink $localtime;  # Important, since it might be a symlink.
     
     &Utils::Report::enter ();
-    $res = symlink ($tz, $localtime);
+    $res = copy ($tz, $localtime);
     &Utils::Report::leave ();
     return -1 unless $res;
     return 0;
@@ -158,6 +158,7 @@ sub get_dist
 {
   my %dist_map =
   (
+   "debian"          => "debian",
    "redhat-6.2"      => "redhat-6.2",
    "redhat-7.0"      => "redhat-6.2",
    "redhat-7.1"      => "redhat-6.2",
@@ -165,8 +166,6 @@ sub get_dist
    "redhat-7.3"      => "redhat-6.2",
    "redhat-8.0"      => "redhat-6.2",
    "mandrake-9.0"    => "redhat-6.2",
-   "debian-3.0"      => "debian-3.0",
-   "ubuntu-7.04"     => "debian-3.0",
    "suse-9.0"        => "redhat-6.2",
    "slackware-9.1.0" => "redhat-6.2",
    "gentoo"          => "redhat-6.2",
@@ -198,17 +197,16 @@ sub conf_get_parse_table
      ]
    },
 
-   "debian-3.0" =>
+   "debian" =>
    {
      fn =>
      {
-       ZONEINFO     => "/usr/share/zoneinfo",
-       LOCAL_TIME   => "/etc/localtime"
+       TIMEZONE     => "/etc/timezone"
      },
      table =>
      [
       [ "local_time",   \&get_utc_time ],
-      [ "timezone",     \&get_timezone, [LOCAL_TIME, ZONEINFO] ],
+      [ "timezone",     \&Utils::Parse::get_first_line, TIMEZONE ],
      ]
    },
 
@@ -267,7 +265,7 @@ sub conf_get_replace_table
      ]
    },
        
-   "debian-3.0" =>
+   "debian" =>
    {
      fn =>
      {
